#!/opt/local/bin/python2.7

#JWST Dictionary
# Version 3.1, Modified 5/18/2015

# This software is provided as is without any warranty whatsoever.
# Permission to use and modify is granted.

#Standard window manager in Python
import Tkinter
from Tkinter import *
import textwrap
import re
import time
import os
import tkFont
import textwrap
import tkMessageBox

# Linux or Mac?
global whatisOS;
whatisOS = os.popen("uname").read().rstrip()

# My version of consumers?
# My version keeps add/edit/delete entry capability
global mine; mine = False

#Defining acrodict_tk to be the Tk class of Tkinter module
class AcroDictApp(Tkinter.Tk):

    # Initialization of window.
    def __init__(self,parent):
        # Tk initialization
        Tkinter.Tk.__init__(self,parent)
        # Keeping track of parent
        self.parent = parent
        # Window title
        self.title('JWST Acronym Dictionary')
        # Set window size, color, and general look
        self.geometry("340x620")
        self.configure(relief=RAISED)
        self.configure(background="Grey")
        # Set global history variable
        self.history = []
        # Set up link exceptions variable (acronyms with mixed case)
        # Need to find someway around this in version 4.
        self.exceptions =['NIRCam','NIRSpec']
        # Set path to acronyms.dat file
        self.datafilepath = os.popen("echo $JWSTACROPATH").read().rstrip()
        # If environment variable does not exist:
        if self.datafilepath == '':
            if whatisOS == 'Darwin': 
                # Set default path for Mac
                self.datafilepath='/Applications/JWST.Acronym.Dictionary.app/Contents/Resources/'
            else:
                duh = tkMessageBox.showwarning("ERROR:","%s is not defined. " \
                                               +"See instructions for help." \
                                         % 'JWSTACROPATH')
                if duh == 'ok':
                    self.destroy()
                    return
        # Verify location of acronyms.dat:
        if os.path.isfile(self.datafilepath+"acronyms.dat"):
            pass
        else:
            duh = tkMessageBox.showwarning("ERROR:" \
                ,"Cannot find acronyms.dat in %s \n\nVerify the environment " \
                +"variable, JWSTACROPATH." \
                % self.datafilepath)
            if duh == 'ok':
                self.destroy()
                return

        # Transparency and background image? (Mac only)
        # Verify exists only if Mac
        if whatisOS == 'Darwin':
            if os.path.isfile(self.datafilepath+'JAD_background.gif'):
                self.backcolor="systemTransparent"
            else:
                print "Didn't find JAD_background.gif in "+self.datafilepath+'.'
                print "Setting background color to white."
                self.backcolor="white"
        # If the OS is Linux or other, just set to white.
        else:
            self.backcolor="white"
            
        # If running in terminal, tell user where looking for acronyms.dat
        print 'Looking for acronyms.dat in '+self.datafilepath

        # MENUBAR (will appear at top of desktop for mac)
        top = Menu(self)                             # attach Menu to window
        self.config(menu=top)                        # cross-link window to menu
        filemenu = Menu(top)                         # attach Menu to top Menu
        filemenu.add_command(label='Exit' \
                             ,command=sys.exit)     # attach option to cascade 
        top.add_cascade(label='File',menu=filemenu) 
        paramenu = Menu(top)                         
        paramenu.add_command(label='Add Entry',command=self.OnAddClick)
        paramenu.add_command(label='Delete Entry',command=self.OnDeleteClick)
        paramenu.add_command(label='Edit Entry',command=self.OnEditClick)
        if mine == False:
            paramenu.entryconfig("Add Entry",state="disabled")
            paramenu.entryconfig("Delete Entry",state="disabled")
            paramenu.entryconfig("Edit Entry",state="disabled")     
        top.add_cascade(label='Edit', menu=paramenu) 
        helpmenu = Menu(top)
        helpmenu.add_command(label='About',command=self.helpwindow)
        top.add_cascade(label='Help',menu=helpmenu)
        
        #Initialize
        self.initialize()

        #----------------------#
        # Tags to Canvas Items #
        #----------------------#
        # have to put it here for it to work. 
        self.db.tag_bind('dbTag','<Button-1>',self.onDbClick)
        self.gbb.tag_bind('gbbTag','<Button-1>',self.onGbbClick)
        self.gfb.tag_bind('gfbTag','<Button-1>',self.onGfbClick)

    #Initializing stuff
    def initialize(self):

        #-------#
        # Fonts #
        #-------#
        # Different fonts needed for Mac and Linux.
        if whatisOS == 'Darwin':
            # Normal result title font
            self.customFont1 = tkFont.Font(family="Times", size=18 \
                                        ,weight="bold")
            # Listbox -> Matches font
            self.customFont2 = tkFont.Font(family="Plantagenet Cherokee", size=14)
            # Grey instructions title font
            self.instructFont1 = tkFont.Font(family="Times",size=18 \
                                         ,weight="bold")
            # Grey instructions details font
            self.instructFont2 = tkFont.Font(family="Times",size=16)
        else:
            # Normal title font
            self.customFont1 = tkFont.Font(family="Nimbus Roman No9 L", size=14 \
                               , weight="bold")
            # Listbox -> Matches font
            self.customFont2 = tkFont.Font(family="DejaVu Serif", size=10 \
                                           ,weight="bold")
            # Grey instructions title font
            self.instructFont1 = tkFont.Font(family="Nimbus Roman No9 L",size=14 \
                                         , weight="bold")
            # Grey instructions details font
            self.instructFont2 = tkFont.Font(family="Nimbus Roman No9 L" \
                                             ,size=12,weight="bold")


        #-------------------------------#
        # Definition and Details window #
        #-------------------------------#
        # Make image background of label and use label as container
        if self.backcolor == "systemTransparent":
            self.tkimage = PhotoImage(file=self.datafilepath+'JAD_background.gif') 
            self.defdetframe=Label(self,image = self.tkimage)
            self.defdetframe.image=self.tkimage
        # If not, make a big white label for container
        else:
            self.defdetframe=Label(self,background=self.backcolor,width=340,height=300)
        self.defdetframe.place(x=0,y=40,width=340,height=300)
       
        # Match title and tracking variable
        global labelVariable; labelVariable = Tkinter.StringVar()
        self.definText = BetterText(self.defdetframe,width=45,wrap=WORD,relief=FLAT \
                          ,borderwidth=0,highlightthickness=0 \
                          ,background=self.backcolor,font=self.customFont1,height=2)
        self.definText.configure(state='normal')
        self.definText.place(x=10,y=10,width=320,height=65)

        # Match result box
        self.detailText = BetterText(self.defdetframe,width=45,wrap=WORD,relief=FLAT \
                               ,borderwidth=0,highlightthickness=0 \
                               ,background=self.backcolor,font=self.instructFont2)
        self.detailText.configure(state='normal')
        self.detailText.place(x=10,y=80,width=320,height=200)

        # Highlight tags
        self.detailText.tag_config('highlight',background='yellow')
        self.definText.tag_config('highlight',background='yellow')

        # Link tags
        self.detailText.tag_config('hyper',foreground="blue",underline=1)
        self.detailText.tag_bind('hyper','<Enter>' \
                                 ,self.detailText.config(cursor="hand2"))
        self.detailText.tag_bind('hyper','<Leave>',self.detailText.config(cursor=""))
        self.detailText.tag_bind('hyper','<Button-1>',self.textClick)
        
        
        # Instructions box.
        instructionsText1 = 'Type any letter to start an acronym search.\n\n'
        instructionsText2 = 'Insert a % before the first letter to search by keyword.'
        self.instructTextBox = BetterText(self.defdetframe,width=31,wrap=WORD \
                                          ,relief=FLAT \
                                    ,borderwidth=20,highlightthickness=0 \
                                    ,background=self.backcolor,foreground="SlateGrey" \
                                    ,font=self.instructFont1)
        self.instructTextBox.insert(0.0,instructionsText1)
        self.instructTextBox.insert(END,instructionsText2)
        self.instructTextBox.tag_config('diffFont',font=self.instructFont2)
        self.instructTextBox.tag_add('diffFont','3.0',END)
        self.instructTextBox.configure(state=DISABLED)

        #-----------#
        # Searchbar #
        #-----------#  
        # Searchbar must be put here so that background image displays right away.
        self.search = SearchBar(0,0,340,40)
        # Place autocomplete entry box on top of search bar canvas object.
        self.entry2 = AutocompleteEntry(self,width=30,relief=FLAT,borderwidth=0 \
                                        ,highlightthickness=0,fg="grey" \
                                        ,background="white")
        self.entry2.insert(0,'>SEARCH')
        self.entry2.bind("<FocusIn>",self.searchDefaultTextRemove)
        self.entry2.place(x=80,y=10,width=230,height=20)
        # Delete entry button/shape (canvas object with tag)
        self.db = Canvas(width=16, height=16,borderwidth=0,highlightthickness=0 \
                         ,background="white")   
        self.db.create_oval(0,0,14,14,fill="grey",outline="grey",tags='dbTag')
        if whatisOS == 'Darwin':
            self.db.create_line(4,4,11,11,fill="white",width=2,tags='dbTag')
            self.db.create_line(4,11,11,4,fill="white",width=2,tags='dbTag')
        else:
            self.db.create_line(3,3,11,11,fill="white",width=2,tags='dbTag')
            self.db.create_line(3,11,11,3,fill="white",width=2,tags='dbTag')

        #------------#
        # Middle Bar #
        #------------#
        # Canvas objects with tags
        x1=0
        x2=340
        y1=340
        y2=360
        self.middlebar = Canvas(width=x2-x1, height=y2-y1,borderwidth=0 \
                                ,highlightthickness=0)
        greywidth = 340
        greyheight = 20
        factor = 150./greyheight
        # Left gradient rectangle
        for offset in range(0, greyheight/2):
            gradColor = '#%02x%02x%02x' % (116.+offset*factor,133.+offset*factor \
                                           ,150.+offset*factor)
            self.middlebar.create_line(0, offset, 40, offset, fill=gradColor)
        for offset in range(greyheight/2,greyheight):
            gradColor = '#%02x%02x%02x' % (266.-offset*factor,283.-offset*factor \
                                           ,300.-offset*factor)
            self.middlebar.create_line(0, offset, 40, offset, fill=gradColor)
        # Middle gradient rectangle
        for offset in range(0, greyheight/2):
            gradColor = '#%02x%02x%02x' % (116.+offset*factor,133.+offset*factor \
                                           ,150.+offset*factor)
            self.middlebar.create_line(60, offset, 280, offset, fill=gradColor)
        for offset in range(greyheight/2,greyheight):
            gradColor = '#%02x%02x%02x' % (266.-offset*factor,283.-offset*factor \
                                           ,300.-offset*factor)
            self.middlebar.create_line(60, offset, 280, offset, fill=gradColor)
        # Right gradient rectangle
        for offset in range(0, greyheight/2):
            gradColor = '#%02x%02x%02x' % (116.+offset*factor,133.+offset*factor \
                                           ,150.+offset*factor)
            self.middlebar.create_line(300, offset, 340, offset, fill=gradColor)
        for offset in range(greyheight/2,greyheight):
            gradColor = '#%02x%02x%02x' % (266.-offset*factor,283.-offset*factor \
                                           ,300.-offset*factor)
            self.middlebar.create_line(300, offset, 340, offset, fill=gradColor)
        # Back arrow background
        for offset in range(0, greyheight):
            gradColor = '#778899' 
            self.middlebar.create_line(280, offset, 300, offset, fill=gradColor) 
        # Forward arrow background
        for offset in range(0, greyheight):
            gradColor = '#778899' 
            self.middlebar.create_line(40, offset, 60, offset, fill=gradColor)        
            #self.middlebar.place(x=x1,y=y1)
        # Search history Go back button  
        self.gbb = Canvas(width=16,height=16,borderwidth=0,highlightthickness=0 \
                          ,background="#778899")
        # Keep track, so can disable/able later.
        self.bk = self.gbb.create_polygon(2,8,14,2,14,14,fill="black",tags='gbbTag')
        # Search history Go forward button
        self.gfb = Canvas(width=16,height=16,borderwidth=0,highlightthickness=0 \
                          ,background="#778899")
        # Keep track, so can disable/able later.
        self.fw = self.gfb.create_polygon(2,2,14,8,2,14,fill="black",tags='gfbTag')

        #-----------------#
        # Results Listbox #
        #-----------------#
        self.resBoxframe = Frame(self,width=340,height=300,background='#778899')
        self.resBoxframe.place(x=0,y=340,width=340,height=280)

        if whatisOS == 'Darwin':
            self.resBox = Listbox(self.resBoxframe,width=42,height=260,relief=FLAT \
                    ,bg='LightGrey',borderwidth=0,highlightthickness=0 \
                    ,font=self.customFont2)
        else:
            # Linux Listbox font is too dark. Need to lighten.
            self.resBox = Listbox(self.resBoxframe,width=42,height=260,relief=FLAT \
                              ,bg='LightGrey',borderwidth=0,highlightthickness=0 \
                              ,font=self.customFont2,foreground="gray16")
        self.scrollbar = Tkinter.Scrollbar(self,command=self.resBox.yview)
        self.resBox.config(yscrollcommand=self.scrollbar.set)
        self.resBox.config(selectbackground='LightSlateGrey',selectforeground='White')
        self.resBox.bind("<<ListboxSelect>>", self.entry2.selection)
        self.resBox.bind("<Return>", self.entry2.selection)
        # Pack in self.resBoxframe
        self.resBox.pack(padx=0,pady=2)        

        #---------------------#
        # Populate dictionary #
        #---------------------#
        self.PopAcroDict()

    #####################
    # OTHER DEFINITIONS #
    #####################
    # What to display if About is chosen from Help in menu bar
    def helpwindow(self):
        tophelp = Tk()
        tophelp.title('About')
        tophelp.geometry("300x225")
        tophelp.configure(background="white")
        # Put simple textbox 
        helptext = BetterText(tophelp,width=30,wrap=WORD,relief=FLAT \
                        ,borderwidth=0,highlightthickness=0 \
                        ,background="white",foreground="Black" \
                        ,height=8 \
                        ,font=tkFont.Font(family="Plantagenet Cherokee" \
                                          , size=14,weight="bold"))
        helpinfo = "\nJWST Acronym Dictionary\nVersion 3.1" \
            "\n\nCreated by Dan. Kiminki\nUniversity of Arizona, 2014" \
            "\n\nLast modified on May 18, 2015\n\n"
        helptext.setText(helpinfo)
        helptext.pack()
        helptext2 = Text(tophelp,width=35,wrap=WORD,relief=FLAT \
                         ,borderwidth=0,highlightthickness=0 \
                         ,background="white",foreground="Black" \
                         ,height=5 \
                         ,font=tkFont.Font(family="Plantagenet Cherokee" \
                                           , size=14,weight="bold"))
        helpinfo2 = "This software is provided as is without any warranty. " \
          "Permission to use and modify is granted."
        helptext2.insert(0.0,helpinfo2)
        helptext2.pack()

    # Action when delete button is clicked
    def onDbClick(self,event):
        # Remove entry
        self.entry2.delete(0,END)
        # set focus on entry
        self.entry2.focus()
        # Reset history and take down forward/back/delete buttons
        self.history = []
        self.db.place_forget()
        self.gbb.place_forget()
        self.gfb.place_forget()
        self.middlebar.place_forget()
        self.middlebar.place(x=0,y=340)
        self.search.place_forget()
        self.search.place(0,0)


    # Action to remove default text in search bar
    def searchDefaultTextRemove(self,event):
        if self.entry2.var.get() == '>SEARCH':
            self.entry2.delete(0,END)
            self.entry2.configure(fg="black")

    # Highlight appropriate text when searching by keyword
    def highlight(self,text_widget, keyword, tag):
        # Code for details section
        pos = '1.0'
        while True:
            idx = text_widget.search(keyword, pos, END,nocase=1)
            if not idx:
                break
            try:
                pos = '{}+{}c'.format(idx, len(keyword))
            except ValueError:
                pos = "%s+%sc" % (idx,len(keyword))
            text_widget.tag_add(tag, idx, pos)

    # ##################################
    # Stuff for clicking links in text #
    ####################################
    # Action when Back button is clicked
    def onGbbClick(self,event):
        # Get entry from results box
        lb_entry = self.resBox.get( \
                    self.resBox.curselection()).replace("\n","").split("->")
        current = lb_entry[0].rstrip().split("->")[0].strip()
        # Get location of entry2 in history
        try:
            curloc = self.history.index(current)
            if curloc > 0:
                back = self.history[curloc-1]
                self.entry2.delete(0,END)
                self.entry2.var.set(back)
                self.entry2.focus_set()
                # Enable forward button
                self.gfb.itemconfig(self.fw,state=NORMAL)
                self.gfb.place(x=40,y=11)
                self.update_idletasks()
            # Disable back button if at first in history
            if self.history.index(back) == 0 or curloc == 0:
                self.gbb.itemconfig(self.fw,state=HIDDEN)
                self.update_idletasks()
        except:
            pass
        
    # Action when Forward button clicked
    def onGfbClick(self,event):
        # Get entry from results box
        lb_entry = self.resBox.get( \
                    self.resBox.curselection()).replace("\n","").split("->")
        current = lb_entry[0].rstrip().split("->")[0].strip()
        # Get location of entry2 in history
        try:
            curloc = self.history.index(current)
            if (curloc+1) < len(self.history):
                forward = self.history[curloc+1]
                self.entry2.delete(0,END)
                self.entry2.var.set(forward)
                self.entry2.focus_set()
                # Enable back button
                self.gbb.itemconfig(self.bk,state=NORMAL)
                self.gbb.place(x=20,y=11)
                self.update_idletasks()
            # Disable forward button if at end of history
            if (curloc+1) == len(self.history)-1:
                self.gfb.itemconfig(self.bk,state=HIDDEN)
                self.update_idletasks()
        except:
            pass

        
    # Check details for acronyms and replace with links to 
    # that acronym. 
    def links(self,text_widget,tag):
        # Get string to search
        stuffToSearch = text_widget.get(0.0,END)
        for key in acros2.keys():
            pos = '1.0'     
            # Lots of IF statements to minimize time cost from re.
            if key not in [x.upper() for x in self.exceptions]:
                # First find out if they key is in the details.
                if key in stuffToSearch:
                    # Now determine if there if it's surrounded by whitespace
                    # or punctuation (to prevent matching problems like HS and HST).
                    match = re.findall('[\)\s.,!?\/\(]%s[\)\s.,!?\/\(]' % key \
                                       ,stuffToSearch)
                    # Now find where those matches are.
                    for item in match:
                        keyword = item.encode('ascii').strip('\s.,!?\/\)\(')
                        pos = '1.0'
                        while True:
                            idx = text_widget.search(keyword,pos,END,nocase=1)
                            if not idx:
                                break
                            try:
                                pos = '{}+{}c'.format(idx,len(keyword))
                            except ValueError:
                                pos = "%s+%sc" % (idx,len(keyword))
                            # Insert hyperlink
                            text_widget.tag_add(tag,idx,pos)
            else:
                # First find out if they key is in the details.
                newkey = self.exceptions[[x.upper() for x in self.exceptions].index(key)]
                if newkey in stuffToSearch:
                    # Now determine if there if it's surrounded by whitespace
                    # or punctuation (to prevent matching problems like HS and HST).
                    match = re.findall('[\)\s.,!?\/\(]%s[\)\s.,!?\/\(]' % newkey \
                                       ,stuffToSearch,re.IGNORECASE) 
                    # Now find where those matches are.
                    for item in match:
                        keyword = item.encode('ascii').strip('\s.,!?\/\)\(')
                        pos = '1.0'
                        while True:
                            idx = text_widget.search(keyword,pos,END,nocase=1)
                            if not idx:
                                break
                            try:
                                pos = '{}+{}c'.format(idx,len(keyword))
                            except ValueError:
                                pos = "%s+%sc" % (idx,len(keyword))
                            # Insert hyperlink
                            text_widget.tag_add(tag,idx,pos)

    # If Links are clicked in details section    
    def textClick(self,event):
        # Get what's being clicked on:
        ranges = self.detailText.tag_ranges('hyper')
        whichRange = self.detailText.index(CURRENT)
        # Got sort through and grab the right link (if there 
        # are multiple of the same one).
        for i in range(0, len(ranges), 2):
            start = ranges[i]
            stop = ranges[i+1]
            nwhichRange = float(str(whichRange).split('.')[1])
            nstart = float(str(start).split('.')[1])
            nstop = float(str(stop).split('.')[1])
            if (nwhichRange >= nstart) and (nwhichRange <= nstop):
                # strip off all crap around match
                tagkey = self.detailText.get(start,stop).encode('ascii').strip().upper()
                # What's the current selection
                lb_entry = self.resBox.get( \
                        self.resBox.curselection()).replace("\n","").split("->")
                current = lb_entry[0].rstrip().split("->")[0].strip()
                # Add current if current is first in history
                if self.history == []:
                    self.history.append(current)
                    # Add next entry
                    self.history.append(tagkey)
                else:
                    # Now add next entry
                    self.history.append(tagkey)
                    # Remove everything after current entry first.
                    # This is in case Go back has been clicked and
                    # then another link clicked. (***the following indices
                    # code is here as a start to rectify when an 
                    # acronym is clicked on again, i.e., you go around
                    # in circles. Can't fix it completely yet***)
                    indices = [i for i, x in enumerate(self.history) if x == tagkey]
                    self.history = self.history[0: indices[-1]+1]
                    # Deactivate forward button
                    self.gfb.place_forget()
                # Remove entry2
                self.entry2.delete(0,END)
                self.update_idletasks()
                self.entry2.var.set(tagkey)
                self.entry2.focus_set()
                # Enable go back button
                self.gbb.itemconfig(self.bk,state=NORMAL)
                self.gbb.place(x=20,y=11)
                self.update_idletasks()


    # ###################################
    # Stuff for Dictionary Modification #
    #####################################    
    # Populate Acronym Dictionary
    def PopAcroDict(self):
        # Populate dictionary, acros2
        global acros2; acros2={}
        with open(self.datafilepath+"acronyms.dat","r") as f:
            data = f.read()
            paragraphs = data.split("\n\n")
            for item in paragraphs:
                try:
                    (keys,vals) = item.split("->")
                    # If key already exists, add to value list
                    if keys.upper() in acros2:
                        acros2[keys.upper()].append(vals)
                    else:
                        acros2[keys.upper()] = [vals]
                except ValueError:
                    pass

    # Write dictionary to file (currently disabled)      
    def WriteAcroDict(self):
        #Sort dictionary and put in list of tuples.
        sortacros2=[ (k,acros2[k]) for k in sorted(acros2.keys())]
        # Open file and overwrite with modified contents.
        with open(self.datafilepath+"acronyms.dat","w") as f:
            for ii in sortacros2:
                for item in ii[1]:
                    f.write(ii[0]+'->')
                    f.write(item+'\n\n')
                    
    # If Edit is chosen from menu (currently disabled)
    def OnEditClick(self):
        # Get entry contents
        editkey = self.resBox.get(self.resBox.curselection()).split("->")
        # Get label contents
        editlb = self.definText.get(0.0,END)
        #editlb = labelVariable.get()
        # Get output box contents
        editout = self.detailText.get(0.0,END).rstrip('\n').encode('ascii')
        # Figure out which dictionary entry this is
        matchvals = acros2[editkey[0].strip()]
        # Find out which entry in dictionary (for multiple matches)
        search = re.compile('\n'+editlb.strip())
        match = [i for i, word in enumerate(matchvals) if re.match(search,word)]
        # Change appropriate dictionary entry
        matchvals[match[0]]='\n'+editlb+' ='+editout
        acros2[editkey[0].strip()] = matchvals
        # Tell user that it has successfully been modified.
        self.definText.setText(editlb+'  has been modified')
        # Write updated dictionary file.
        self.WriteAcroDict()
        # Re-populate dictionary
        self.PopAcroDict()

    # If Add is chosen from menu (currently disabled)
    def OnAddClick(self):
        #Get entry variable input and add to dictionary
        search = self.entry2.var.get().strip().split(' ')
        newkey = search[0].rstrip() #<=== in case user provides two words
        newval = self.detailText.get(0.0,END).encode('ascii').rstrip('\n')
        # in case user doesn't follow directions
        if newval != self.entry2.instructions and len(newval) != 0:
            if len(newkey) > 1:
                # Find out if already in dictionary
                if newkey.upper() in acros2:
                    # Append dictionary value list
                    acros2[newkey.upper()].append('\n'+newval)
                else:
                    #Add new key and value.
                    acros2[newkey.upper()]=['\n'+newval]  
                #Write updated dictionary to file.
                self.WriteAcroDict()
                #Re-populate dictionary.
                self.PopAcroDict()             
                # Update labels.
                self.definText.setText(newkey.upper()+' added to dictionary.')
                # Focus back on entry variable
                self.entry2.focus_set()
                self.entry2.var.set(newkey)
                # Clean up textbox
                self.detailText.delete(0.0,Tkinter.END)
            else:
                # Clean up textbox
                self.detailText.delete(0.0,Tkinter.END)
                # Alert user to invalidity
                self.definText.setText('Not a valid entry')
                self.detailText.insert(0.0,self.entry2.instructions)
        else:
            # Clean up textbox
            self.detailText.delete(0.0,END)
            # Alert user to invalidity 
            self.instructTextBox.place_forget()
            self.definText.setText('Not a valid entry')
            self.detailText.insert(0.0,self.entry2.instructions)

    # If delete is chosen from menu (currentely disabled)
    def OnDeleteClick(self):
        #Get entry to remove.
        entryToDel = self.definText.get(0.0,END)
        #Get all entries matching acronym.
        acroEntryToDel = self.resBox.get(ACTIVE).split("->")[0].strip()
        if acroEntryToDel in acros2:
            allEntries = acros2[acroEntryToDel]
            #If there is more than one match, remove the right one.
            if len(allEntries) > 1:
                for item in allEntries:
                    item2 = item.split('=')
                    if item2[0].strip() == entryToDel:
                        allEntries.remove(item)
                        acros2[self.entry2.var.get()]=allEntries
            #If there is only one match, remove it.
            elif len(allEntries) == 1:
                del acros2[self.resBox.get(ACTIVE).split("->")[0].strip()]
            self.definText.setText(entryToDel+' has been removed')  
        else:
            self.definText.setText('No entry for '+self.entry2.var.get())  

        #Clean up boxes.
        self.detailText.delete(0.0,Tkinter.END)

        #Write updated dictionary to file.
        self.WriteAcroDict()
        
        #Re-populate dictionary
        self.PopAcroDict()

#%%%%%%%%%%%%%%%%%%%%
# --OTHER CLASSES-- %
#%%%%%%%%%%%%%%%%%%%%
class SearchBar:
    
    # Search bar definition 
    # (would be better if this was less hardcoded for position)
    def __init__(self,x1,y1,x2,y2, *args, **kwargs):
        # Use canvas to draw searchbar
        self.searchbar = Canvas(width=x2-x1, height=y2-y1,borderwidth=0 \
                                ,highlightthickness=0)
        greywidth = x2-x1
        greyheight = y2-y1
        # Gradient rectangle.
        factor = 150./greyheight
        for offset in range(0, greyheight/2):
            gradColor = '#%02x%02x%02x' % (116.+offset*factor,133.+offset*factor \
                                           ,150.+offset*factor)
            self.searchbar.create_line(x1, y1 + offset, x1+greywidth, y1 + offset \
                                       , fill=gradColor)
        for offset in range(greyheight/2, greyheight):
            gradColor = '#%02x%02x%02x' % (266.-offset*factor,283.-offset*factor \
                                           ,300.-offset*factor)
            self.searchbar.create_line(x1, y1 + offset, x1+greywidth, y1 + offset \
                                       , fill=gradColor)
        # Curved grey shapes behind entry widget
        self.searchbar.create_rectangle(x1+20,y1+6,x2-20,y1+32 \
                                        ,fill="#778899",outline="#778899")
        self.searchbar.create_oval(x1+8,y1+6,x1+32,y1+32,fill="#778899" \
                                   ,outline="#778899")
        self.searchbar.create_oval(x2-28,y1+6,x2-8,y1+32,fill="#7b8c9d" \
                                   ,outline="#778899")

        # Curved white shapes behind entry widget
        self.searchbar.create_rectangle(x1+70, y1+8, x2-20, y1+30 \
                                        , fill="white",outline="white")
        self.searchbar.create_oval(x1+60,y1+8,x1+80,y1+30,fill="white",outline="white")
        self.searchbar.create_oval(x2-30,y1+8,x2-10,y1+30,fill="white",outline="white")
        self.searchbar.place(x=x1,y=y1)

    def place_forget(self):
        self.searchbar.place_forget()

    def place(self,x,y):
        self.searchbar.place(x=x,y=y)
        
        
class BetterText(Text):
    def __init__(self, *args, **kwargs):

        #Initialize
        Text.__init__(self, *args, **kwargs)

        #Tags
        self.tag_configure('sometag', justify='center')

    def setText(self,newtext):
        self.delete(0.0,END)
        self.insert(0.0,newtext)
        self.tag_add('sometag',1.0,END)

               
# The general layout of the autocomplete code is based on that found here:
# http://code.activestate.com/recipes/578253-an-entry-with
#-autocompletion-for-the-tkinter-gui/
        
class AutocompleteEntry(Entry):
    def __init__(self, *args, **kwargs):
        
        Entry.__init__(self, *args, **kwargs)
        self.var = self["textvariable"]
        if self.var == '':
            self.var = self["textvariable"] = StringVar()

        self.var.trace('w', self.changed)
        self.bind("<Return>",self.selection)
        self.bind("<Up>", self.up)
        self.bind("<Down>", self.down)

        self.focus_set()
        self.resBox_up = False

        # Contribute an entry instructions
        self.instructions="To suggest an entry, please contribute using the dictionary webpage:" \
          +"\n\nhttp://deggial.as.arizona/edu/jad/submit.php"

    # When some form of input is received (typed letter, delete, etc)
    def changed(self, name, index, mode): 
        # Clean up output box
        self.master.detailText.delete(0.0,END) 
        self.master.definText.delete(0.0,END)
        # What to do if searchbar has nothing or default SEARCH string
        if self.var.get() == '' or self.var.get() == '>SEARCH':
            # If listbox (results) has entries, remove them,
            # take down detailText (details) and label (definition).
            if self.resBox_up != False:
                self.master.resBox.delete(0,END)
                self.resBox_up = False                
            # Remove definition (label) and details (output)
            self.master.definText.place_forget()
            self.master.detailText.place_forget()
            # Try/Except here for startup because objects don't exist yet
            try:
                # Remove forward/back/delete buttons
                self.master.db.place_forget()
            except AttributeError:
                pass
            # Put up instructions
            self.master.instructTextBox.place_forget()
            self.update()
            self.master.instructTextBox.place(x=10,y=80,height=200)

        # If searchbar has text (not default)
        else:
            # Take down instructions
            self.master.instructTextBox.place_forget()
            # Put up delete entry button
            self.master.db.place(x=310,y=12)
            # Put up detailText (details) and label (definition)
            self.master.detailText.place_forget()
            self.master.definText.place_forget()
            # Have to do an update or the above placement stuff doesn't
            # happen until after loop (i.e., things get drawn over and
            # it's not pretty).
            self.update()
            self.master.detailText.place(x=10,y=80,width=320,height=200)
            self.master.definText.place(x=10,y=10,width=320,height=65)
            # Display matches, if they exist
            words = self.comparison()
            if words:  
                self.resBox_up = True
                # Fill in matches in results listbox
                self.master.resBox.delete(0, END)
                for w in sorted(words):
                    self.master.resBox.insert(END,w)
                # See if there is an exact match and select if exist
                if self.get().upper() in acros2:
                    # grab key-definition and dust off all the extra crap.
                    exact = self.get().upper()+' -> ' \
                      +acros2[self.get().upper()][0].strip("\n").split('=')[0].rstrip(' ')
                    # get location in results listbox
                    exactloc = words.index(exact)
                    # select that location
                    self.master.resBox.selection_clear(0,END)
                    self.master.resBox.selection_set(exactloc)
                    self.master.resBox.activate(exactloc)
                    # Scroll to selection
                    self.master.resBox.yview(exactloc)
                #If no exact match, then set selection to first item.
                else:
                    self.master.resBox.selection_clear(first='0')
                    self.master.resBox.selection_set(first='0')
                    self.master.resBox.activate('0')
                # What does the following do again??
                self.selection("<Button-1>")
            else:
                # Remove results listbox entries
                self.master.resBox.delete(0,END)
                # Set listbox tracker (nothing in it now).
                self.resBox_up = False
                

    # Separate dictionary entries
    def SepDictComps(self,DictEntry):
        DictEntrySep = {}
        #Separate entry
        if "=" in DictEntry:
            restemp = DictEntry.split("=")
            res1 = restemp[0].strip()
            res2 = restemp[1]
            res2 = "\n".join(textwrap.wrap(res2,50))
        else:
            res1 = 'No description'
            res2 = DictEntry
        DictEntrySep[res1] = res2                
        return DictEntrySep

    # Get selection and display
    def selection(self, event):
        
        # Do hightlight of matches
        if self.get() != '':
            if self.get()[0] == '%':
                hlightmatch = self.get().replace("%","")
            else:
                hlightmatch = self.get()


        # If autocomplete listbox has content:                        
        if self.resBox_up:
            # If there is no selected entry:
            if self.master.resBox.curselection() == ():
                lb_entry = []
                lb_entry.append(self.get())
            # If there IS a selected entry:
            else:
                # Get selected entry
                lb_entry = self.master.resBox.get( \
                                self.master.resBox.curselection()).replace("\n","").split("->")
                self.icursor(END)

            # Now, populate textbox and label
            search = lb_entry[0].rstrip().split("->")
            search_acro = search[0].strip()
            try:
                result = acros2[search_acro.upper()]
                # If focus is on entry box, choose first match
                if len(lb_entry) < 2:
                    match = 0
                # else, choose which ever is selected
                else:
                    # If two entries, get the one chosen.
                    index = 0
                    for item in result:
                        item2 = item.split('=')
                        if item2[0].strip() == lb_entry[1].strip():
                            match = index
                        index += 1
                # Separate entry
                dictentries = self.SepDictComps(result[match])
                # Put results in label and textbox.
                self.master.detailText.place_forget()
                self.master.definText.place_forget()
                self.update()
                self.master.detailText.place(x=10,y=80,width=320,height=200)
                self.master.definText.place(x=10,y=10,width=320,height=65)
                self.master.definText.setText(''.join(dictentries.keys()))
                self.master.detailText.delete(0.0,Tkinter.END)
                self.master.detailText.insert(1.0
                    ,''.join(dictentries.values()).replace("\n"," "))
                
                # Highlight matches
                self.master.highlight(self.master.detailText,hlightmatch,'highlight')
                self.master.highlight(self.master.definText,hlightmatch,'highlight')

                # Link if links exist
                self.master.links(self.master.detailText,"hyper")

            # If selection is not in dictionary, then show how to add
            # entry.
            except KeyError:
                # If not present, let user know.
                self.master.definText.setText('Not in Dictionary')
                #Clean up textbox 
                self.master.detailText.delete(0.0,Tkinter.END)
                #Instructions on how to add
                self.master.instructTextBox.place_forget()
                self.master.detailText.insert(0.0,self.instructions)
                #self.resBox_up = False
                self.icursor(END)
        # If list box is not up (i.e., no matches), then show
        # how to add entry.
        else:
            # If not present, let user know.
            self.master.definText.setText('Not in Dictionary')
            #Clean up textbox 
            self.master.detailText.delete(0.0,Tkinter.END)
            #Instructions on how to add
            self.master.instructTextBox.place_forget()
            self.master.detailText.insert(0.0,self.instructions)

    # If up arrow key is pressed.
    def up(self, event):
        if self.resBox_up:
            if self.master.resBox.curselection() == ():
                index = '-1'
            else:
                index = self.master.resBox.curselection()[0]
            if int(index) >= -1: 
                self.master.resBox.selection_clear(first=index)
                index = str(int(index)-1)                
                self.master.resBox.selection_set(first=index)
                self.master.resBox.activate(index) 
                self.master.resBox.yview(index)
        # this command is to prevent \ut700 from being expressed 
        #in entry widget.
        return "break"

    # If down arrow key is pressed.
    def down(self, event):
        if self.resBox_up:
            if self.master.resBox.curselection() == ():
                index = '0'                
            else:
                index = self.master.resBox.curselection()[0]
            if index != self.master.resBox.size()-1:                        
                self.master.resBox.selection_clear(first=index)
                index = str(int(index)+1)        
                self.master.resBox.selection_set(first=index)
                self.master.resBox.activate(index) 
                self.master.resBox.yview(index)
        # this command is to prevent \ut701 from being expressed 
        #in entry widget.
        return "break"

    # Get matches to search input
    def comparison(self):
        search_entry = self.var.get().split("->")
        if search_entry[0][0] == '%':
            search_dict = search_entry[0].replace("%","").replace("(","\(").replace(")","\)").replace(".","\.")
            # Remove escape stuff
            matches = []
            if len(search_dict) > 0:
                for i in range(len(acros2.values())):
                    for j in range(len(acros2.values()[i])):
                        insensitive=re.compile(search_dict, re.IGNORECASE)
                        match=re.search(insensitive, acros2.values()[i][j])
                        if match:
                            match_val_split = acros2.values()[i][j].replace("\n","").split("=")
                            matches.append(acros2.keys()[i]+" -> "+match_val_split[0])
                return matches
        else:
            search_dict = search_entry[0].replace("(","\(").replace(")","\)").replace(".","\.")
            search = re.compile('.*' + search_dict.rstrip().upper() + '.*') 
            matchlist=[]
            matches=[]
            for i in range(len(acros2.keys())):
                match = re.search(search,acros2.keys()[i])
                if match:
                    matchlist.append(acros2.keys()[i])
            for w in sorted(matchlist):
                for item in acros2[w]:
                    matches.append(w+' -> '+''.join(item).split("=")[0].strip())
            return matches


#=========> START GUI <==========
#This is executed if run from command line
if __name__=="__main__":

    #Call acrodict_tk with no parent (None)
    app = AcroDictApp(None)
    #Start an infinite loop
    app.mainloop()

    

